{***************************************************************************}
{*                                                                         *}
{*                              ANTICOPY UNIT                              *}
{*                              version 1.101                              *}
{*                        Author : Oleg I. Subachev                        *}
{*                        Zavodskaya St., 45 A - 11                        *}
{*                   V-109, Yekaterinburg, 620109, Russia                  *}
{*                      home phone: +7 (3432) 42-27-49                     *}
{*                        e-mail: soi@urvb.e-burg.su                       *}
{*                                                                         *}
{*     Copyright (C) 1992,93 by Oleg I. Subachev, Yekaterinburg, Russia    *}
{*                                                                         *}
{***************************************************************************}

(*
Version history :
-----------------

Version 1.0 : - first version of AntiCopy unit.

Version 1.1 : - ability to install permitted number of copies
                ( according to proposal from Vera Noest
                  Vera.Noest@vaxtbio.uu.se );
              - TP 7.0 version of AntiCopy unit was included
                in the package;
              - some style improvements and more comments were
                added in source code.

Version 1.101 : - my e-mail address has changed 8-)
*)

{$A-,B-,E-,G+,I-,N-,R-,S-,V-}
{$F+,O+}
{$D-,L-}

unit ANTICOPY;

interface

uses
  Dos;

implementation

procedure COPY_RIGHT;
var
  sCR : string;
begin
  sCR := ' AntiCopy 1.1 is copyright (C) 1992,93 by Oleg I. Subachev,'+
         ' Yekaterinburg, Russia ';
end;

{ $DEFINE CHK_DOS_300}          { version of DOS must not be less than 3.00 }
{$DEFINE WIPE}     { protected .EXE file will be wiped out, not only halted }

{$DEFINE BIOS_DT}                               { BIOS date will be checked }
{$DEFINE FULL_PATH}  { full start path of protected program will be checked }
{ $DEFINE IDE_INF}         { IDE HDD controller information will be checked }
{ $DEFINE VOLSERNUM}                 { volume serial number will be checked }
{ $DEFINE BOOT_SEC}               { boot sector of drive C: will be checked }
{ $DEFINE EQUIPMENT}        { equipment word from BIOS data will be checked }
{ $DEFINE START_DR}      { start drive of protected program will be checked }

{ $DEFINE KEEPDTTM}             { keep date and time of protected .EXE file }
{ $DEFINE PERMIT_N}                    { some number of copies is permitted }
                  { if PERMIT_N is defined, then protected .EXE file cannot }
                     { be compressed by utility such as LZEXE, PKLITE, DIET }
                                    { and so on, even after first execution }

{ $DEFINE CHK_PASS}                   { exit procedure, that checks passing }
                                 { through ANTICOPY unit, will be installed }

{$IFDEF BIOS_DT}
{ the main symptom - BIOS date }
const
  ac_BIOS_DT
    : word = 0;                                                 { BIOS date }
var
  wBIOS_DT
    : word;
{$ENDIF}

{$IFDEF FULL_PATH}
const
  ac_FLLPTH                        { full start path of protected .EXE file }
    : word = 0;
var
  wFLLPTH
    : word;
{$ENDIF}

{$IFDEF IDE_INF}
const
  ac_IDECIN                                { IDE HDD controller information }
    : longint = 0;
var                                      { this the best indicator to check }
  lIDECIN       { because every IDE HDD controller has unique serial number }
    : longint;  { unfortunately this check will hang computer with SCSI HDD }
{$ENDIF}

{$IFDEF VOLSERNUM}
const
  ac_VLSRNM                                          { volume serial number }
    : longint = 0;
var
  lVLSRNM
    : longint;
{$ENDIF}

{$IFDEF BOOT_SEC}
const
  ac_BT_SEC                                       { boot sector information }
    : longint = 0;
var
  lBT_SEC
    : longint;
{$ENDIF}

{$IFDEF EQUIPMENT}
const
  ac_EQPMNT                                 { equipment word from BIOS data }
    : word = 0;
{$ENDIF}

{$IFDEF START_DR}
const
  ac_STRTDR                               { protected .EXE file start drive }
    : byte = 0;
var
  bySTRTDR
    : byte;
{$ENDIF}

{$IFDEF PERMIT_N}
const
  iPQOCOP
    : integer = 1; { <- permitted quantity of copies }
{$ENDIF}

var
  PS0                                                     { = ParamStr( 0 ) }
    : PathStr;
  f                                            { protected .EXE file itself }
    : file;

{$IFDEF CHK_PASS}
  ac_SavedExitProc
    : pointer;
{$ENDIF}

{$IFDEF VOLSERNUM}
  {$L AC-UNIT2.OBJ}
  function ReadDiskSectors( Drive : Word;
                            FirstSect : Longint;
                            NumSects : Word;
                            var Buf ) : Boolean; far; external;
{$ELSE}
  {$IFDEF BOOT_SEC}
    {$L AC-UNIT2.OBJ}
    function ReadDiskSectors( Drive : Word;
                              FirstSect : Longint;
                              NumSects : Word;
                              var Buf ) : Boolean; far; external;
  {$ENDIF}
{$ENDIF}

procedure NA_FIG; { <- the name of procedure is russian slang }
{ optionally wipes protected .EXE file and halts program }
var
  L : longint;
begin                                                              { NA_FIG }
{  WriteLn( 'Illegal copy was detected' );}

{$IFDEF CHK_PASS}
  ExitProc := ac_SavedExitProc;               { if we are here, we have not }
                                                   { to execute ac_ExitProc }
{$ENDIF}

{$IFDEF WIPE}
  Seek( f, 0 );                            { wiping out protected .EXE file }
  for L := 1 to FileSize( f ) div 16
    do BlockWrite( f, L, 16 );
{$ENDIF}

  Close( f );

{ you do not have to erase the protected file, because user
  will try to unerase it anyhow, 8-)
  but you *may* erase

  Erase( f );
}

  Halt;
end;                                                               { NA_FIG }

procedure DETERMINE;
{ determines information for checking }
var
  W : word;
  pBOOT_SECTOR : pointer;

begin                                                           { DETERMINE }

{ NOTs are added to "encrypt" information }

{$IFDEF BIOS_DT}
  wBIOS_DT := not ( MemW[ $F000:$FFF5 ]
                  + MemW[ $F000:$FFF8 ]
                  + MemW[ $F000:$FFFB ] );
{$ENDIF}

{$IFDEF FULL_PATH}
  wFLLPTH := 0;
  for W := 1 to Length( PS0 )
    do Inc( wFLLPTH, Ord( PS0[ W ] ));
{$ENDIF}

{$IFDEF IDE_INF}                                          { *** WARNING *** }
                              { if you will use this checking on a computer }
                                         { that has not IDE HDD controller, }
                                                    { the computer may hang }
  lIDECIN := 0;
  while ( Port[ $1F7 ] <> $50 ) do;          { wait for controller not busy }
  Port[ $1F6 ] := $A0;                                 { select first drive }
  Port[ $1F7 ] := $EC;                                { get drive info data }
  while ( Port[ $1F7 ] <> $58 ) do;                   { wait for data ready }
  for W := 0 to 255                   { read IDE HDD controller information }
    do Inc( lIDECIN, PortW[ $1F0 ]);
{$ENDIF}

{$IFDEF VOLSERNUM}
  GetMem( pBOOT_SECTOR, 512 );
  if not ReadDiskSectors( 2, 0, 1, pBOOT_SECTOR^ ) { reading of BOOT sector }
    then NA_FIG;
{$ELSE}
  {$IFDEF BOOT_SEC}
    GetMem( pBOOT_SECTOR, 512 );
    if not ReadDiskSectors( 2, 0, 1, pBOOT_SECTOR^ )
      then NA_FIG;
  {$ENDIF}
{$ENDIF}

{$IFDEF VOLSERNUM}
{ DOSes with version less than 4.00 don't support volume serial number }
  if Swap( DosVersion ) >= $0400
    then lVLSRNM := not ( MemL[ Seg( pBOOT_SECTOR^ )
                              : Ofs( pBOOT_SECTOR^ ) + $27 ] )
    else lVLSRNM := 0;
{$ENDIF}

{$IFDEF BOOT_SEC}
  lBT_SEC := 0;
  for W := 0 to 511
    do Inc( lBT_SEC, Mem[ Seg( pBOOT_SECTOR^ )
                        : Ofs( pBOOT_SECTOR^ ) + W ] );
{$ENDIF}

{$IFDEF VOLSERNUM}
  FreeMem( pBOOT_SECTOR, 512 );
{$ELSE}
  {$IFDEF BOOT_SEC}
    FreeMem( pBOOT_SECTOR, 512 );
  {$ENDIF}
{$ENDIF}

{$IFDEF START_DR}
  bySTRTDR := not ( Ord( PS0[ 1 ] ));
{$ENDIF}

(*
other information that may be checked:

- CMOS values,
- file length,
- control sum of file,
- file name,
- size of current drive,
- current directory,
- first cluster of file,
- partition table,
- DOS version,
- presence of some drivers (such as QEMM386.SYS)
  and their serial numbers (if they have ones)

and so on
*)

end;                                                            { DETERMINE }

procedure ZAPIS; { <- russian for "writing" }
{ writes information for checking in file f }
var
  wHDR_SIZ            { size of header of protected .EXE file in paragraphs }
    : word;

  procedure ac_SEEK( lFIL_OFF : longint );
  { moves current position in f to lFIL_OFF }
  begin
    if lFIL_OFF >= FileSize( f )                     { this condition means }
      then NA_FIG;                      { that *before* the first execution }
                            { protected .EXE file was compressed by utility }
                                    { such as LZEXE, PKLITE, DIET and so on }
                                         { ANTICOPY can not treat such file }
                          { but *after* first execution protected .EXE file }
                                   { may be compressed and all will work OK }
                             { this condition will not recognize compressed }
                              { file, if it has overlays and was compressed }
                                  { *before* the first execution, such file }
                                                     { can not be protected }

    Seek( f, lFIL_OFF );
  end;                                                            { ac_SEEK }

  function FASP( var TC ) : longint;
  { "magic" 8-) function that returns file offset of typed constant TC }
  begin
    FASP := longint( Seg( TC ) - PrefixSeg + wHDR_SIZ ) * $10
                   + Ofs( TC ) - $100;
  end;                                                               { FASP }

var
  lDT                                   { date and time for Get- Set- FTime }
    : longint;

begin                                                               { ZAPIS }
  ac_SEEK( $8 );                { determination of wHDR_SIZ for use in FASP }
  BlockRead( f, wHDR_SIZ, SizeOf( wHDR_SIZ ));

{$IFDEF KEEPDTTM}
  GetFTime( f, lDT );       { save date and time of the protected .EXE file }
{$ENDIF}

{ writing of new values of typed constants into protected .EXE file }

{$IFDEF BIOS_DT}
  ac_SEEK( FASP( ac_BIOS_DT ));
  BlockWrite( f, wBIOS_DT, SizeOf( wBIOS_DT ));
{$ENDIF}

{$IFDEF FULL_PATH}
  ac_SEEK( FASP( ac_FLLPTH ));
  BlockWrite( f, wFLLPTH, SizeOf( wFLLPTH ));
{$ENDIF}

{$IFDEF IDE_INF}
  ac_SEEK( FASP( ac_IDECIN ));
  BlockWrite( f, lIDECIN, SizeOf( lIDECIN ));
{$ENDIF}

{$IFDEF VOLSERNUM}
  ac_SEEK( FASP( ac_VLSRNM ));
  BlockWrite( f, lVLSRNM, SizeOf( lVLSRNM ));
{$ENDIF}

{$IFDEF BOOT_SEC}
  ac_SEEK( FASP( ac_BT_SEC ));
  BlockWrite( f, lBT_SEC, SizeOf( lBT_SEC ));
{$ENDIF}

{$IFDEF EQUIPMENT}
  ac_SEEK( FASP( ac_EQPMNT ));
  BlockWrite( f, MemW[ 0 : $410 ], SizeOf( word ));
{$ENDIF}

{$IFDEF START_DR}
  ac_SEEK( FASP( ac_STRTDR ));
  BlockWrite( f, bySTRTDR, SizeOf( bySTRTDR ));
{$ENDIF}

{$IFDEF PERMIT_N}
  ac_SEEK( FASP( iPQOCOP ));
  Dec( iPQOCOP );
  BlockWrite( f, iPQOCOP, SizeOf( iPQOCOP ));
{$ENDIF}

{$IFDEF KEEPDTTM}
  SetFTime( f, lDT );    { restore date and time of the protected .EXE file }
{$ENDIF}

(* this code may be included to halt program after writing checking
   information during first execution

  Close( f );
  ac_CHK_AC := true;
  Halt;
*)

end;                                                                { ZAPIS }

{$IFDEF CHK_PASS}
const
  ac_CHK_AC                { is used to check passing through ANTICOPY unit }
    : boolean = false;

procedure ac_ExitProc; far;
begin
  ExitProc := ac_SavedExitProc;
  if ac_CHK_AC                    { this check provides that we have passed }
    then Close( f )                                 { through ANTICOPY unit }
    else NA_FIG;
end;                                                          { ac_ExitProc }
{$ENDIF}

begin                                                                { MAIN }

{$IFDEF CHK_DOS_300}            { If version of DOS is less than 3.00, then }
  if Swap( DosVersion ) < $0300 then Exit;  { ParamStr( 0 ) will not return }
{$ENDIF}           { correct value ( I even don't know what it will return, }
                            { because I have never worked with DOS < 3.00 ) }
                                 { But as DOSes with version < 3.00 usually }
                          { are not used, you may not check this condition. }
                                               { Usually, I don't check it. }

{$IFDEF CHK_PASS}
  ac_SavedExitProc := ExitProc;
  ExitProc := @ac_ExitProc;
{$ENDIF}

  PS0 := ParamStr( 0 );
  Assign( f, PS0 );
  Reset( f, 1 );

  DETERMINE;

  if

{$IFDEF BIOS_DT}
     ac_BIOS_DT            
{$ELSE}                    
  {$IFDEF FULL_PATH}       
       ac_FLLPTH
  {$ELSE}
    {$IFDEF IDE_INF}
         ac_IDECIN
    {$ELSE}
      {$IFDEF VOLSERNUM}
           ac_VLSRNM
      {$ELSE}
        {$IFDEF BOOT_SEC}
             ac_BT_SEC
        {$ELSE}
          {$IFDEF START_DR}
               ac_STRTDR
          {$ELSE}
            {$IFDEF EQUIPMENT}
                 ac_EQPMNT
            {$ENDIF}
          {$ENDIF}
        {$ENDIF}
      {$ENDIF}
    {$ENDIF}
  {$ENDIF}                     
{$ENDIF}                          { if first declared ac_... typed constant }
                                                         { is equal to zero }
     = 0                   { (the init value of all ac_... typed constants) }
    then ZAPIS                             { then it is the first execution }

    else                                                         { checking }
      if

{$IFDEF BIOS_DT}
         ( ac_BIOS_DT <> wBIOS_DT )
{$ELSE}
           False
{$ENDIF}

{$IFDEF FULL_PATH}
       or ( ac_FLLPTH <> wFLLPTH )
{$ENDIF}

{$IFDEF IDE_INF}
       or ( ac_IDECIN <> lIDECIN )
{$ENDIF}

{$IFDEF VOLSERNUM}
       or ( ac_VLSRNM <> lVLSRNM )
{$ENDIF}

{$IFDEF BOOT_SEC}
       or ( ac_BT_SEC <> lBT_SEC )
{$ENDIF}

{$IFDEF START_DR}
       or ( ac_STRTDR <> bySTRTDR )
{$ENDIF}

{$IFDEF EQUIPMENT}
       or ( ac_EQPMNT <> MemW[ 0 : $410 ] )
{$ENDIF}

{$IFDEF PERMIT_N}
        then
          if iPQOCOP >= 0
            then ZAPIS
               else NA_FIG;
{$ELSE}
        then NA_FIG;
{$ENDIF}

{$IFDEF CHK_PASS}
  ac_CHK_AC := true;
{$ENDIF}

end.
